@@ -165,8 +165,8 @@ mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 
 	dialog->p->dialog_win = gtk_dialog_new_with_buttons
 		(_("Edit file type"), NULL, -1,
-		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 NULL);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->p->dialog_win)->vbox), WID ("edit_widget"), TRUE, TRUE, 0);
@@ -264,6 +264,8 @@ mime_edit_dialog_set_prop (GObject *object, guint prop_id, const GValue *value,
 		if (mime_edit_dialog->p->is_add) {
 			mime_edit_dialog->p->info = mime_type_info_new (NULL, NULL);
 			setup_add_dialog (mime_edit_dialog);
+			gtk_window_set_title (GTK_WINDOW (mime_edit_dialog->p->dialog_win),
+				(_("Add file type")));
 			gtk_widget_show_all (mime_edit_dialog->p->dialog_win);
 		}
 
@@ -356,14 +358,31 @@ mime_add_dialog_new (GtkTreeModel *model)
 			     NULL);
 }
 
+static void
+safe_set_entry (MimeEditDialog *dialog, char const *widget, char const *txt)
+{
+	GtkEntry *entry =  GTK_ENTRY (WID (widget));
+
+	g_return_if_fail (entry != NULL);
+
+	if (txt == NULL)
+		txt = "";
+	gtk_entry_set_text (entry, txt);
+}
+
 static void
 fill_dialog (MimeEditDialog *dialog)
 {
+	g_return_if_fail (dialog->p->info != NULL);
+
 	mime_type_info_load_all (dialog->p->info);
 
-	gtk_entry_set_text (GTK_ENTRY (WID ("description_entry")), dialog->p->info->description);
-	gtk_entry_set_text (GTK_ENTRY (WID ("mime_type_entry")), dialog->p->info->mime_type);
-	gtk_entry_set_text (GTK_ENTRY (WID ("category_entry")), mime_type_info_get_category_name (dialog->p->info));
+	safe_set_entry (dialog, "description_entry",
+		dialog->p->info->description);
+	safe_set_entry (dialog, "mime_type_entry",
+		dialog->p->info->mime_type);
+	safe_set_entry (dialog, "category_entry",
+		mime_type_info_get_category_name (dialog->p->info));
 
 	dialog->p->use_cat_dfl = dialog->p->info->use_category;
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("use_category_defaults_toggle")), dialog->p->use_cat_dfl);
@@ -409,7 +428,7 @@ populate_component_list (MimeEditDialog *dialog)
 	GtkWidget         *menu_item;
 	GtkOptionMenu     *component_select;
 	gchar             *component_name;
-	Bonobo_ServerInfo *info;
+	Bonobo_ServerInfo *info, *default_component;
 	int                found_idx = -1, i;
 
 	menu = GTK_MENU (gtk_menu_new ());
@@ -419,10 +438,14 @@ populate_component_list (MimeEditDialog *dialog)
 	/* FIXME: We are leaking the whole list here, but this will be the case until I know of an easy way to duplicate
 	 * Bonobo_ServerInfo structures */
 
+	default_component = dialog->p->info->default_component;
 	for (tmp = component_list, i = 0; tmp != NULL; tmp = tmp->next, i++) {
 		info = tmp->data;
 
-		if (!strcmp (info->iid, dialog->p->info->default_component->iid))
+		g_return_if_fail (info != NULL);
+
+		if (default_component != NULL &&
+		    !strcmp (info->iid, default_component->iid))
 			found_idx = i;
 
 		component_name = mime_type_get_pretty_name_for_server (info);
@@ -624,6 +647,8 @@ store_data (MimeEditDialog *dialog)
 		if (!mime_type_info_using_custom_app (dialog->p->info)) {
 			gnome_vfs_mime_application_free (dialog->p->info->default_action);
 			dialog->p->info->default_action = g_new0 (GnomeVFSMimeApplication, 1);
+		} else if (!dialog->p->info->default_action) {
+			dialog->p->info->default_action = g_new0 (GnomeVFSMimeApplication, 1);
 		}
 
 		g_free (dialog->p->info->default_action->command);
@@ -646,7 +671,7 @@ store_data (MimeEditDialog *dialog)
 	}
 	g_free (tmp);
 
-	model_entry_append_to_dirty_list (MODEL_ENTRY (dialog->p->info));
+	model_entry_save (MODEL_ENTRY (dialog->p->info));
 
 	if (!cat_changed) {
 		mime_types_model_construct_iter (MIME_TYPES_MODEL (dialog->p->model),
@@ -661,34 +686,30 @@ static gboolean
 validate_data (MimeEditDialog *dialog) 
 {
 	const gchar *tmp;
-	GtkWidget *err_dialog;
+	const gchar *mesg = NULL;
 
 	tmp = gtk_entry_get_text (GTK_ENTRY (WID ("mime_type_entry")));
 
 	if (tmp != NULL && *tmp != '\0') {
 		if (strchr (tmp, ' ') || !strchr (tmp, '/')) {
-			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->p->dialog_win),
-							     0, GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_OK,
-							     _("Invalid MIME type. Please enter a valid MIME type, or "
-							       "leave the field blank to have one generated for you."));
-
-			gtk_window_set_modal (GTK_WINDOW (err_dialog), TRUE);
-
-			return FALSE;
-		}
-		else if (dialog->p->is_add && (gnome_vfs_mime_type_is_known (tmp) || get_mime_type_info (tmp) != NULL)) {
-			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->p->dialog_win),
-							     0, GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_OK,
-							     _("There already exists a MIME type of that name."));
-
-			gtk_window_set_modal (GTK_WINDOW (err_dialog), TRUE);
-
-			return FALSE;
+			mesg = _("Invalid MIME type. Please enter a valid MIME type, or "
+				 "leave the field blank to have one generated for you.");
+		} else if (dialog->p->is_add && (gnome_vfs_mime_type_is_known (tmp) ||
+						 get_mime_type_info (tmp) != NULL)) {
+			mesg = _("There already exists a MIME type of that name.");
 		}
 	}
 
+	if (mesg != NULL) {
+		GtkWidget *err_dialog = gtk_message_dialog_new (
+			GTK_WINDOW (dialog->p->dialog_win),
+			GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+			GTK_BUTTONS_OK,
+			mesg);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_object_destroy (GTK_OBJECT (err_dialog));
+		return FALSE;
+	}
 	return TRUE;
 }
 
@@ -751,8 +772,8 @@ choose_cat_cb (MimeEditDialog *dialog)
 
 	dialog_win = gtk_dialog_new_with_buttons
 		(_("Choose a file category"), NULL, -1,
-		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 NULL);
 
 	gtk_widget_set_size_request (dialog_win, 300, 300);
